----------------------------------------------------------------------------------
-- Design Name:    NTSC to VGA converter TOP Module
-- Module Name:    NTSC2D1
-- 2010-02-19 Y.Fujioka
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity NTSC2D1 is
    Port (  
			nRESET     		: in std_logic;
			sysclk    		: in std_logic;  -- 27MHz			
	 		Y_in     		: in std_logic_vector(7 downto 0);
			blk_in   		: in std_logic;
			hs_in      		: in std_logic;
			vs_in      		: in std_logic;	
			smooth_on_in  	: in std_logic;
			vga_r_out 		: out std_logic_vector(7 downto 0);
			vga_g_out 		: out std_logic_vector(7 downto 0);
			vga_b_out 		: out std_logic_vector(7 downto 0);					
			vga_blk_out 	: out std_logic;
			vga_hsync_out  : out std_logic; 
	      vga_vsync_out  : out std_logic;
			vga_pclk_out   : out std_logic;
			state_out      : out std_logic_vector(2 downto 0));
end NTSC2D1;

architecture Behavioral of NTSC2D1 is

component YUV422_444_cnv
	port (
	reset				: in	std_logic;
	sysclk			: in	std_logic;
	D_DIN				: in	std_logic_vector(7 downto 0);
	valid_in	   	: in	std_logic;
	y_out				: out	std_logic_vector(7 downto 0);
	u_out				: out	std_logic_vector(7 downto 0);
	v_out				: out	std_logic_vector(7 downto 0);
	valid_out		: out	std_logic);
end component;


component YUV_RGB_cnv
	port(
	RESET				: in std_logic;
	sysclk			: in std_logic;
	DATA_IN_Y		: in std_logic_vector(7 downto 0);
	DATA_IN_U		: in std_logic_vector(7 downto 0);
	DATA_IN_V		: in std_logic_vector(7 downto 0);
	valid_in			: in std_logic;
	DATA_OUT_R		: out std_logic_vector(7 downto 0);
	DATA_OUT_G		: out std_logic_vector(7 downto 0);
	DATA_OUT_B		: out std_logic_vector(7 downto 0);
	valid_out		: out std_logic	
	);
end component;

component UPSCAN_D1_cnv
	port(
	reset      		: in std_logic;
	sysclk     		: in std_logic;  -- 27MHz
	smooth_on   	: in std_logic;			
	r_data_in  		: in std_logic_vector(7 downto 0);
	g_data_in  		: in std_logic_vector(7 downto 0);
	b_data_in  		: in std_logic_vector(7 downto 0);
	valid_in      	: in std_logic;
	hsync_in    	: in std_logic;
	vsync_in    	: in std_logic;	

	vga_r_out 		: out std_logic_vector(7 downto 0);
	vga_g_out 		: out std_logic_vector(7 downto 0);
	vga_b_out 		: out std_logic_vector(7 downto 0);
	vga_blk_out  	: out std_logic;
	vga_hsync_out  : out std_logic; 
	vga_vsync_out  : out std_logic;
	vga_pclk_out  	: out std_logic;
	state_out      : out std_logic_vector(2 downto 0));	 
end component;


signal y_out_node			: std_logic_vector(7 downto 0);
signal u_out_node			: std_logic_vector(7 downto 0);
signal v_out_node			: std_logic_vector(7 downto 0);
signal valid_out_node_1 : std_logic;
signal valid_out_node_2 : std_logic;
signal ntsc_r_node 		:std_logic_vector(7 downto 0);
signal ntsc_g_node 		:std_logic_vector(7 downto 0);
signal ntsc_b_node 		:std_logic_vector(7 downto 0);
signal data_out_node 	: std_logic;
signal reset 				: std_logic;
signal hd    				: std_logic;
signal vd    				: std_logic;
signal blk   				: std_logic;
signal yuv_in 				: std_logic_vector(7 downto 0);
signal smooth_on 			: std_logic;


begin
 
YUV422_444_cnv_1 : YUV422_444_cnv port map (
		reset				=> reset,
		sysclk			=> sysclk,
		D_DIN				=> yuv_in,
		valid_in	   	=> blk,
		y_out				=> y_out_node,
		u_out				=> u_out_node,
		v_out				=> v_out_node,
		valid_out		=> valid_out_node_1);
 
YUV_RGB_cnv1 : YUV_RGB_cnv	port map(
		RESET				=> reset,
		sysclk			=> sysclk,
		DATA_IN_Y		=> y_out_node,
		DATA_IN_U		=> u_out_node,
		DATA_IN_V		=> v_out_node,
		valid_in			=> valid_out_node_1,
		DATA_OUT_R		=> ntsc_r_node,
		DATA_OUT_G		=> ntsc_g_node,
		DATA_OUT_B		=> ntsc_b_node,
		valid_out 		=> valid_out_node_2);	
		

UPSCAN_D1_cnv1 : UPSCAN_D1_cnv port map(
		reset      		=> reset,
		sysclk     		=> sysclk,
		smooth_on   	=> smooth_on,
 		r_data_in  		=> ntsc_r_node,
		g_data_in  		=> ntsc_g_node,
		b_data_in  		=> ntsc_b_node,
		valid_in    	=> valid_out_node_2,
		hsync_in   		=> hd,
		vsync_in    	=> vd,
		vga_r_out  		=> vga_r_out,
      vga_g_out		=> vga_g_out,
		vga_b_out 		=> vga_b_out,
		vga_blk_out  	=> vga_blk_out,
		vga_hsync_out  => vga_hsync_out, 
      vga_vsync_out  => vga_vsync_out,
		vga_pclk_out   => vga_pclk_out,
		state_out      => state_out);	 


process (sysclk)
begin
  if (sysclk'event and sysclk='1') then
--		reset <= '0';	
		reset <= not nRESET;	
	   hd    <= not hs_in;
	   vd    <= not vs_in;
		blk   <= blk_in;
		yuv_in  		<= Y_in;
		smooth_on 	<= '1';
		smooth_on 	<= smooth_on_in;
  end if;
end process;  
	

			
end Behavioral;			

