----------------------------------------------------------------------------------
-- Design Name:    NTSC to VGA Scan Converter UPSCAN Module
-- Module Name:    UPSCAN_D1_CNV
-- 2010-02-04 Y.Fujioka
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity UPSCAN_D1_CNV is
    Port (  
			reset				: in std_logic;
			sysclk			: in std_logic;  -- 27MHz
			smooth_on		: in std_logic;
	 		r_data_in		: in std_logic_vector(7 downto 0);
			g_data_in		: in std_logic_vector(7 downto 0);
			b_data_in		: in std_logic_vector(7 downto 0);

			valid_in			: in std_logic;
			hsync_in    	: in std_logic;
			vsync_in    	: in std_logic;	 

			vga_r_out		: out std_logic_vector(7 downto 0);
			vga_g_out		: out std_logic_vector(7 downto 0);
			vga_b_out		: out std_logic_vector(7 downto 0);
			
			vga_blk_out		: out std_logic;
			vga_hsync_out	: out std_logic; 
	      vga_vsync_out	: out std_logic;
			vga_pclk_out	: out std_logic;
			state_out		: out std_logic_vector(2 downto 0));	
end UPSCAN_D1_CNV;

architecture RTL of UPSCAN_D1_CNV is

component vram_720x8 IS
	port (
	addra	: in std_logic_VECTOR(9 downto 0);
	addrb	: in std_logic_VECTOR(9 downto 0);
	clka	: in std_logic;
	clkb	: in std_logic;
	dina	: in std_logic_VECTOR(7 downto 0);
	doutb	: out std_logic_VECTOR(7 downto 0);
	enb	: in std_logic;
	wea	: in std_logic);
END component;

signal  vga_r_temp		: std_logic_vector (8 downto 0);
signal  vga_g_temp		: std_logic_vector (8 downto 0);
signal  vga_b_temp		: std_logic_vector (8 downto 0);
signal  lb_r_a    		: std_logic_vector (7 downto 0);
signal  lb_g_a    		: std_logic_vector (7 downto 0);
signal  lb_b_a    		: std_logic_vector (7 downto 0);
signal  lb_r_b    		: std_logic_vector (7 downto 0);
signal  lb_g_b    		: std_logic_vector (7 downto 0);
signal  lb_b_b    		: std_logic_vector (7 downto 0);
signal  lb_r_c    		: std_logic_vector (7 downto 0);
signal  lb_g_c    		: std_logic_vector (7 downto 0);
signal  lb_b_c    		: std_logic_vector (7 downto 0);
signal  w_adrs    		: std_logic_vector (9 downto 0);
signal  r_adrs    		: std_logic_vector (9 downto 0);
signal  state     		: std_logic_vector (2 downto 0);
signal  prev_state		: std_logic_vector (2 downto 0);
signal  w_en_a    		: std_logic;
signal  w_en_b    		: std_logic;
signal  w_en_c    		: std_logic;
signal  h_counter 		: std_logic_vector(10 downto 0);
signal  v_counter 		: std_logic_vector(9 downto 0);
signal  vga_hs    		: std_logic;
signal  vga_vs    		: std_logic;
signal  vga_blk   		: std_logic;
signal  vga_h_blk   		: std_logic;
signal  vga_v_blk   		: std_logic;
signal  vga_hs_delay 	: std_logic;
signal  vga_hs_P_edge	: std_logic;
signal  vsync_delay 		: std_logic;
signal  hsync_delay  	: std_logic_vector(2 downto 0);
signal  v_counter_reset : std_logic;
signal  hsync_N_edge 	: std_logic;
signal  vsync_P_edge 	: std_logic;
signal  vsync_hs_delay  : std_logic_vector(1 downto 0);
signal smooth_on_A 		: std_logic;

begin

vram_720x8_r_a: vram_720x8 port map (
	addra => w_adrs,
	addrb => r_adrs,
	clka  => sysclk,
	clkb  => sysclk,
	dina  => r_data_in,
	doutb => lb_r_a,
	enb   => '1',
	wea   => w_en_a
	);
	
vram_720x8_g_a: vram_720x8 port map (
	addra => w_adrs,
	addrb => r_adrs,
	clka  => sysclk,
	clkb  => sysclk,
	dina  => g_data_in,
	doutb => lb_g_a,
	enb   => '1',
	wea   => w_en_a
	);

vram_720x8_b_a: vram_720x8 port map (
	addra => w_adrs,
	addrb => r_adrs,
	clka  => sysclk,
	clkb  => sysclk,
	dina  => b_data_in,
	doutb => lb_b_a,
	enb   => '1',
	wea   => w_en_a
	);	
 

vram_720x8_r_b: vram_720x8 port map (
	addra => w_adrs,
	addrb => r_adrs,
	clka  => sysclk,
	clkb  => sysclk,
	dina  => r_data_in,
	doutb => lb_r_b,
	enb   => '1',
	wea   => w_en_b
	);
	
vram_720x8_g_b: vram_720x8 port map (
	addra => w_adrs,
	addrb => r_adrs,
	clka  => sysclk,
	clkb  => sysclk,
	dina  => g_data_in,
	doutb => lb_g_b,
	enb   => '1',
	wea   => w_en_b
	);

vram_720x8_b_b: vram_720x8 port map (
	addra => w_adrs,
	addrb => r_adrs,
	clka  => sysclk,
	clkb  => sysclk,
	dina  => b_data_in,
	doutb => lb_b_b,
	enb   => '1',
	wea   => w_en_b
	);	
 
vram_720x8_r_c: vram_720x8 port map (
	addra => w_adrs,
	addrb => r_adrs,
	clka  => sysclk,
	clkb  => sysclk,
	dina  => r_data_in,
	doutb => lb_r_c,
	enb   => '1',
	wea   => w_en_c
	);
	
vram_720x8_g_c: vram_720x8 port map (
	addra => w_adrs,
	addrb => r_adrs,
	clka  => sysclk,
	clkb  => sysclk,
	dina  => g_data_in,
	doutb => lb_g_c,
	enb   => '1',
	wea   => w_en_c
	);

vram_720x8_b_c: vram_720x8 port map (
	addra => w_adrs,
	addrb => r_adrs,
	clka  => sysclk,
	clkb  => sysclk,
	dina  => b_data_in,
	doutb => lb_b_c,
	enb   => '1',
	wea   => w_en_c
	);	
 


process (sysclk)
begin
  if (sysclk'event and sysclk='1') then
	  vsync_delay    <= vsync_in;
	  hsync_delay(2) <= hsync_delay(1);
     hsync_delay(1) <= hsync_delay(0);
	  hsync_delay(0) <= hsync_in;	  
     vga_hs_delay <= vga_hs;	  
  end if;
end process;  

hsync_N_edge 	<= (not hsync_in) and hsync_delay(0);
vsync_P_edge 	<= (vsync_in) and (not vsync_delay);
vga_hs_P_edge 	<= vga_hs and (not vga_hs_delay);

-- VSync ̗オHSync HL `FbN邪A͐MɂĂ̓GbWω̂߁AHSync x点

process (sysclk)
begin
  if (sysclk'event and sysclk='1') then
     if ((vsync_P_edge = '1') and (hsync_delay(2) = '0')) then
		     v_counter_reset <= '1';
		  else
		     v_counter_reset <= '0';
	  end if;
  end if;
end process;

process (sysclk)
begin
  if (sysclk'event and sysclk='1') then
     if(vga_hs_P_edge = '1') then
		   vsync_hs_delay(0) <= vsync_in;
			vsync_hs_delay(1) <= vsync_hs_delay(0);
     end if;
  end if;
end process;

-- VGA Timing Generator Description

process (sysclk)
begin
  if (sysclk'event and sysclk='1') then
     if (v_counter_reset = '1') then
				v_counter<=(others=>'0');
		end if;	
   	if ((h_counter=857) or (hsync_N_edge = '1')) then
  		   h_counter<=(others=>'0');		
			if (v_counter=524) then
				v_counter<=(others=>'0');			 	
			else
				v_counter<=v_counter+1;	
			end if;  		
		else 
		h_counter<=h_counter+1;		
		end if;
  end if;
end process;

--Timing Signal Generation Description

vga_blk_out   	<= vga_blk;
vga_hsync_out 	<= vga_hs; 
vga_vsync_out 	<= vga_vs;
vga_pclk_out  	<= sysclk;	 
state_out 		<= state;
vga_hs 			<= '0' when (h_counter >=0 and h_counter < 72) else '1';
vga_h_blk 		<= '0' when (( h_counter >= 0 and h_counter < 120) or ( h_counter >= 840 ))  else '1';
vga_v_blk 		<= '0' when ((v_counter >=0 and v_counter < 34) or (v_counter >= 514)) else '1';
vga_blk 			<= vga_h_blk or vga_v_blk;
vga_vs 			<= '0' when (v_counter >=20 and v_counter < 22) else '1';
smooth_on_A 	<= smooth_on;

-- Line Buffer Read Address Generator

process (sysclk)
begin 
  if (sysclk'event and sysclk='1') then
	 if(vga_h_blk = '0')  then
	 	 r_adrs <= (others => '0');
	 else	 
	    r_adrs <= r_adrs +1;
    end if;
  end if;
end process;

-- Line Buffer Write Address Generator

process (sysclk)
begin
  if (sysclk'event and sysclk ='1') then
    if ((hsync_in = '0') or (w_adrs = 719)) then
	    	w_adrs <= (others => '0');
    elsif(valid_in = '1') then
	    w_adrs <= w_adrs +1;
	 end if;
  end if;
end process;

-- Line Buffer Management State Machine Description
  
process (sysclk)
 begin
  if (sysclk'event and sysclk='1') then
    if(v_counter_reset ='1') then
	    state <= "000";
    elsif (vga_hs_P_edge = '1') then
		    state <= prev_state;   
	 end if;
  end if;
end process;
 
process (state, valid_in, lb_r_c, lb_g_c, lb_b_c, lb_r_b, lb_g_b, lb_b_b, lb_r_a, lb_g_a, lb_b_a, smooth_on_A)
begin
  case state is
    when "000" =>
	   w_en_a <= valid_in;
	   w_en_b <= '0';
	   w_en_c <= '0';	
		if (smooth_on_A = '1') then
			vga_r_temp <= ('0' & lb_r_c) + ('0' & lb_r_b);
			vga_g_temp <= ('0' & lb_g_c) + ('0' & lb_g_b);
			vga_b_temp <= ('0' & lb_b_c) + ('0' & lb_b_b);
		else
			vga_r_temp <= ('0' & lb_r_c) + ('0' & lb_r_c);
			vga_g_temp <= ('0' & lb_g_c) + ('0' & lb_g_c);
			vga_b_temp <= ('0' & lb_b_c) + ('0' & lb_b_c);
		end if;
		prev_state <= "001";

	when "001" =>
	   w_en_a <= valid_in;
	   w_en_b <= '0';
	   w_en_c <= '0';	
		vga_r_temp <= ('0' & lb_r_c) + ('0' & lb_r_c);
		vga_g_temp <= ('0' & lb_g_c) + ('0' & lb_g_c);
		vga_b_temp <= ('0' & lb_b_c) + ('0' & lb_b_c);
		prev_state <= "010";	   
		
	when "010" =>
	   w_en_a <= '0';
	   w_en_b <= valid_in;
	   w_en_c <= '0';
		if (smooth_on_A = '1') then
			vga_r_temp <= ('0' & lb_r_a) + ('0' & lb_r_c);
			vga_g_temp <= ('0' & lb_g_a) + ('0' & lb_g_c);
			vga_b_temp <= ('0' & lb_b_a) + ('0' & lb_b_c);
		else
			vga_r_temp <= ('0' & lb_r_a) + ('0' & lb_r_a);
			vga_g_temp <= ('0' & lb_g_a) + ('0' & lb_g_a);
			vga_b_temp <= ('0' & lb_b_a) + ('0' & lb_b_a);		
		end if;
		prev_state <= "011";	   

	when "011" =>
	   w_en_a <= '0';
	   w_en_b <= valid_in;
	   w_en_c <= '0';	
      vga_r_temp <= ('0' & lb_r_a) + ('0' & lb_r_a);
      vga_g_temp <= ('0' & lb_g_a) + ('0' & lb_g_a);
      vga_b_temp <= ('0' & lb_b_a) + ('0' & lb_b_a);		
		prev_state <= "100";	   
	   
	when "100" =>
	   w_en_a <= '0';
	   w_en_b <= '0';
	   w_en_c <= valid_in;
		if (smooth_on_A = '1') then 
			vga_r_temp <= ('0' & lb_r_b) + ('0' & lb_r_a);
			vga_g_temp <= ('0' & lb_g_b) + ('0' & lb_g_a);
			vga_b_temp <= ('0' & lb_b_b) + ('0' & lb_b_a);		
		else
			vga_r_temp <= ('0' & lb_r_b) + ('0' & lb_r_b);
			vga_g_temp <= ('0' & lb_g_b) + ('0' & lb_g_b);
			vga_b_temp <= ('0' & lb_b_b) + ('0' & lb_b_b);					
		end if;
		prev_state <= "101";	   
	   		 
	when "101" =>
	   w_en_a <= '0';
	   w_en_b <= '0';
	   w_en_c <= valid_in;	
      vga_r_temp <= ('0' & lb_r_b) + ('0' & lb_r_b);
      vga_g_temp <= ('0' & lb_g_b) + ('0' & lb_g_b);
      vga_b_temp <= ('0' & lb_b_b) + ('0' & lb_b_b);		
		prev_state <= "000";

   when others => null;
  end case;
end process;  

vga_r_out(7 downto 0)  <= vga_r_temp(8 downto 1);
vga_g_out(7 downto 0)  <= vga_g_temp(8 downto 1);
vga_b_out(7 downto 0)  <= vga_b_temp(8 downto 1);

end RTL;

