----------------------------------------------------------------------------------
-- Design Name:    from YUV to RGB component
-- Module Name:    YUV2RGB_EQUATION
--  2010-04-19 Y.Fujioka
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity yuv2rgb_equation is
	Port (
		RESET				: in std_logic;
		sysclk			: in std_logic;
		valid_in			: in std_logic;
		DATA_IN_Y		: in std_logic_vector(7 downto 0);
		DATA_IN_U		: in std_logic_vector(7 downto 0);
		DATA_IN_V		: in std_logic_vector(7 downto 0);
		CONST1			: in std_logic_vector(15 downto 0);
		CONST2			: in std_logic_vector(15 downto 0);
		CONST3			: in std_logic_vector(15 downto 0);
		CONST4			: in std_logic_vector(15 downto 0);
		valid_out		: out std_logic;
		DATA_OUT			: out std_logic_vector(7 downto 0)
	);
	end yuv2rgb_equation;


architecture Behavioral of yuv2rgb_equation is
	
signal DATA_OUT_1 	: std_logic_vector(23 DOWNTO 0);
signal DATA_OUT_2 	: std_logic_vector(23 DOWNTO 0);
signal DATA_OUT_3 	: std_logic_vector(23 DOWNTO 0);
signal temp_sum   	: std_logic_vector(23 DOWNTO 0);
	
-- Hard Macro Multiplier in XC3S250E (Unsigned_8 x Signed_16 -> Signed_24)

component yuv2rgb_mult   
	port (
	clk		: in std_logic;
	a			: in std_logic_VECTOR(7 downto 0);
	b			: in std_logic_VECTOR(15 downto 0);
	p			: out std_logic_VECTOR(23 downto 0));
end component;
	
begin
	process (sysclk)
	begin
	 if (sysclk'event and sysclk = '1') then
		if (RESET = '1') then
			valid_out <= '0';
		else
			valid_out	<= valid_in;
		end if;
	end if;	
end process;
	
-- Y Part
	Mult1 : yuv2rgb_mult
	port map (
		clk => sysclk,
		a => DATA_IN_Y,
		b => CONST1,
		p => DATA_OUT_1);
		
-- U Part
	Mult2 : yuv2rgb_mult
	port map (
		clk => sysclk,	
		a => DATA_IN_U,
		b => CONST2,
		p => DATA_OUT_2);
		
-- V Part
	Mult3 : yuv2rgb_mult
	port map (
		clk => sysclk,	
		a => DATA_IN_V,
		b => CONST3,
		p => DATA_OUT_3);

temp_sum <= DATA_OUT_1 + DATA_OUT_2 + DATA_OUT_3 + (CONST4(15 downto 0) & x"00");
DATA_OUT	<=	x"00" when temp_sum(21) = '1' else
				x"ff" when temp_sum(20) = '1' else
				temp_sum (19 downto 12);
	
end Behavioral;
