----------------------------------------------------------------------------------
-- Design Name:    NTSC to VGA converter YUV4:2:2: to 4:4:4
-- Module Name:    yuv422_444_cnv
-- 2010-04-19 Y.Fujioka
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity yuv422_444_cnv is
port (
	reset		   : in	std_logic;
	sysclk		: in	std_logic;

	D_DIN		   : in	std_logic_vector(7 downto 0);
	valid_in 	: in	std_logic;
	
	y_out			: out	std_logic_vector(7 downto 0);
	u_out			: out	std_logic_vector(7 downto 0);
	v_out			: out	std_logic_vector(7 downto 0);
	valid_out	: out	std_logic
);
end yuv422_444_cnv;

architecture Behavioral of yuv422_444_cnv is
	
	constant S0 : std_logic_vector (1 downto 0) := "00";
	constant S1 : std_logic_vector (1 downto 0) := "01";
	constant S2 : std_logic_vector (1 downto 0) := "10";
	constant S3 : std_logic_vector (1 downto 0) := "11";

    signal state 		: std_logic_vector(1 downto 0);	
    signal data_in 		: std_logic_vector(7 downto 0);	
	signal u_reg_1	 	: std_logic_vector(7 downto 0);
	signal v_reg_1	 	: std_logic_vector(7 downto 0);	
	signal y0_reg_1	 	: std_logic_vector(7 downto 0);
	signal y1_reg_1	 	: std_logic_vector(7 downto 0);	
	signal u_reg_2	 	: std_logic_vector(7 downto 0);
	signal v_reg_2	 	: std_logic_vector(7 downto 0);	
	signal y0_reg_2	 	: std_logic_vector(7 downto 0);
	signal y1_reg_2	 	: std_logic_vector(7 downto 0);	
    signal data_out_en  : std_logic;
 
     signal valid_out_node  : std_logic;
     signal y_out_node  : std_logic_vector(7 downto 0);
     signal u_out_node  : std_logic_vector(7 downto 0);    
     signal v_out_node  : std_logic_vector(7 downto 0);       
        

	
begin



	
process (sysclk)
begin
  if (sysclk'event and sysclk = '1') then
   if (reset = '1' or valid_in = '0') then
 	   state <= S0;
 	   y0_reg_2 <= (others=>'0');
       y1_reg_2 <= (others=>'0');
       u_reg_2 <= (others=>'0');
       v_reg_2 <= (others=>'0');		
	   data_out_en <= '0'; 
	else
	    if (state = S0) then
 	       u_reg_1 <= data_in;
	       state <= state +1; 
        elsif (state = S1) then
           y0_reg_1 <= data_in;
           state <= state +1; 
	    elsif (state = S2) then
           v_reg_1 <= data_in;
           state <= state +1; 
 	    elsif (state = S3) then    
           y0_reg_2 <= y0_reg_1;
           y1_reg_2 <= data_in;
           u_reg_2 <= u_reg_1;
           v_reg_2 <= v_reg_1;
           data_out_en <= '1';
           state <= S0;
      end if;
    end if;
 end if;
end process;

data_in <= D_DIN;
  valid_out <= '1' when (data_out_en = '1' and (state = S0 or state = S2)) else '0';
  y_out <= y0_reg_2 when (state = S0 or state = S1) else y1_reg_2;
  u_out <= u_reg_2;
  v_out <= v_reg_2;            



	
	
end Behavioral;
