----------------------------------------------------------------------------------
-- Design Name:    NTSC to VGA converter YUV to RGB
-- Module Name:    yuv_rgb_cnv
-- 2010-04-19 :    Y.Fujioka
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity yuv_rgb_cnv is
Port (
	RESET			: in std_logic;
	sysclk		: in std_logic;
	DATA_IN_Y	: in std_logic_vector(7 downto 0);
	DATA_IN_U	: in std_logic_vector(7 downto 0);
	DATA_IN_V	: in std_logic_vector(7 downto 0);
	valid_in		: in std_logic;
	DATA_OUT_R	: out std_logic_vector(7 downto 0);
	DATA_OUT_G	: out std_logic_vector(7 downto 0);
	DATA_OUT_B	: out std_logic_vector(7 downto 0);
	valid_out	: out std_logic
);
end yuv_rgb_cnv;

architecture Behavioral of yuv_rgb_cnv is

-- Constant for R
constant Y_CONST_R	: std_logic_vector(15 downto 0)	:= x"12A0";	-- Y
constant U_CONST_R	: std_logic_vector(15 downto 0)	:= x"0000";	-- Cb
constant V_CONST_R	: std_logic_vector(15 downto 0)	:= x"18d5";	-- Cr
constant D_CONST_R	: std_logic_vector(15 downto 0)	:= x"f211";	-- Const

-- Constant for G
constant Y_CONST_G	: std_logic_vector(15 downto 0)	:= x"12A0";	-- Y
constant U_CONST_G	: std_logic_vector(15 downto 0)	:= x"f9ba";	-- Cb
constant V_CONST_G	: std_logic_vector(15 downto 0)	:= x"f2fe";	-- Cr
constant D_CONST_G	: std_logic_vector(15 downto 0)	:= x"087a";	-- Const

-- Constant for B
constant Y_CONST_B	: std_logic_vector(15 downto 0)	:= x"12A0";	-- Y
constant U_CONST_B  : std_logic_vector(15 downto 0)	:= x"2046";	-- Cb
constant V_CONST_B	: std_logic_vector(15 downto 0)	:= x"0000";	-- Cr
constant D_CONST_B	: std_logic_vector(15 downto 0)	:= x"eeb3";	-- Const
	
signal valid_out_R : std_logic;
signal valid_out_G : std_logic;
signal valid_out_B : std_logic;	


component yuv2rgb_equation
Port (
	RESET			: in std_logic;
	sysclk		: in std_logic;
	valid_in		: in std_logic;
	DATA_IN_Y	: in std_logic_vector(7 downto 0);
	DATA_IN_U	: in std_logic_vector(7 downto 0);
	DATA_IN_V	: in std_logic_vector(7 downto 0);
	CONST1		: in std_logic_vector(15 downto 0);
	CONST2		: in std_logic_vector(15 downto 0);
	CONST3		: in std_logic_vector(15 downto 0);
	CONST4		: in std_logic_vector(15 downto 0);
	valid_out	: out std_logic;
	DATA_OUT		: out std_logic_vector(7 downto 0)
);
end component;

begin

RGB_CONV_R: yuv2rgb_equation
port map (
	RESET			=> RESET,
	sysclk		=> sysclk,
	valid_in		=> valid_in,
	DATA_IN_Y	=> DATA_IN_Y,
	DATA_IN_U	=> DATA_IN_U,
	DATA_IN_V	=> DATA_IN_V,
	CONST1		=> Y_CONST_R,
	CONST2		=> U_CONST_R,
	CONST3		=> V_CONST_R,
	CONST4		=> D_CONST_R,
	valid_out	=> valid_out_R,
	DATA_OUT		=> DATA_OUT_R
);

RGB_CONV_G: yuv2rgb_equation
port map (
	RESET			=> RESET,
	sysclk		=> sysclk,
	valid_in		=> valid_in,
	DATA_IN_Y	=> DATA_IN_Y,
	DATA_IN_U	=> DATA_IN_U,
	DATA_IN_V	=> DATA_IN_V,
	CONST1		=> Y_CONST_G,
	CONST2		=> U_CONST_G,
	CONST3		=> V_CONST_G,
	CONST4		=> D_CONST_G,
	valid_out	=> valid_out_G,
	DATA_OUT		=> DATA_OUT_G
);

RGB_CONV_B: yuv2rgb_equation
port map (
	RESET			=> RESET,
	sysclk		=> sysclk,
	valid_in		=> valid_in,
	DATA_IN_Y	=> DATA_IN_Y,
	DATA_IN_U	=> DATA_IN_U,
	DATA_IN_V	=> DATA_IN_V,
	CONST1		=> Y_CONST_B,
	CONST2		=> U_CONST_B,
	CONST3		=> V_CONST_B,
	CONST4		=> D_CONST_B,
	valid_out	=> valid_out_B,
	DATA_OUT		=> DATA_OUT_B	
	);

valid_out <= valid_out_R;

end Behavioral;
